package com.microservices.pms.specialArea.mapper;

import com.microservices.common.datasource.annotation.Master;
import com.microservices.pms.specialArea.domain.ForgeProject;
import com.microservices.pms.specialArea.domain.ZonePmsProject;
import com.microservices.pms.specialArea.domain.vo.KeyValVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Master
@Mapper
public interface SoftwareFactStatisPmsMapper {

    List<ZonePmsProject> selectPmsProjectList();

    ZonePmsProject selectPmsProjectById(@Param("id") Long id);
    // 状态 0待开始 1进行中 2已关闭
    List<KeyValVO> countSprintByStatus(Long projectId);
    Long countSprintByCUES(ForgeProject forgeProject);
    Long countTestCaseBy(Long projectId);
    Long countTestCaseInSheet(ForgeProject forgeProject);
    Long countTestCaseByCUES(ForgeProject forgeProject);

    String selectPmsUserByGitlinkId(@Param("gitlinkId") Long gitlinkId);

    List<Long> selectPmsProjectListByDict();

}
